package org.mekelburger.moritz.scanble;

import android.util.Log;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

/**
 * This class stores a history of RSSI values of received advertisements with time stamps
 */
public class AdvertisementHistory extends HashMap<Long, Integer> {
    private final static int OLD = 3000;

    public AdvertisementHistory() {
        super();
    }

    /**
     * Add an RSSI value to the data base. It will be stored together with the current time stamp.
     *
     * @param rssi
     */
    public void addAdvertisement(int rssi) {
        this.put(new Long(System.currentTimeMillis()), new Integer(rssi));
    }

    /**
     * Calculate the mean value of the latest values. Old entries (more then
     * AdvertisementHistory.OLD ms old) are deleted.
     *
     * @return average value of latest RSSI values, Returns NaN if there are no recent RSSI values
     */
    public float getAverageValue() {
        float result = 0.f;
        long now = System.currentTimeMillis();
        Set<Long> recent = new HashSet<Long>();

        for (Long ts: this.keySet()) {
            if (now - ts.longValue() < OLD) {
                recent.add(ts);
                result += this.get(ts).longValue();
            }
        }
        this.keySet().retainAll(recent);
        return result / (float) this.size();
    }

    public float getError() {
        float result = 0.f;
        float m = this.getAverageValue();
        for (Integer v: this.values()) {
            result += Math.pow(v.doubleValue() - m, 2);
        }
        return result/(this.size() - 1);
    }
}
